#!/usr/bin/env python
#-*- coding: UTF-8 -*-

"""
This module uses the sample.py script to load all test models it finds.

Note: this is not an exhaustive test suite, it does not check the
data structures in detail. It just verifies whether basic
loading and querying of 3d models using pyassimp works.
"""

import os
import sys

# Make the development (ie. GIT repo) version of PyAssimp available for import.
sys.path.insert(0, '..')

import sample
from pyassimp import errors

# Paths to model files.
basepaths = [os.path.join('..', '..', '..', 'test', 'models'),
             os.path.join('..', '..', '..', 'test', 'models-nonbsd')]

# Valid extensions for 3D model files.
extensions = ['.3ds', '.x', '.lwo', '.obj', '.md5mesh', '.dxf', '.ply', '.stl',
              '.dae', '.md5anim', '.lws', '.irrmesh', '.nff', '.off', '.blend']


def run_tests():
    ok, err = 0, 0
    for path in basepaths:
        print("Looking for models in %s..." % path)
        for root, dirs, files in os.walk(path):
            for afile in files:
                base, ext = os.path.splitext(afile)
                if ext in extensions:
                    try:
                        sample.main(os.path.join(root, afile))
                        ok += 1
                    except errors.AssimpError as error:
                        # Assimp error is fine; this is a controlled case.
                        print(error)
                        err += 1
                    except Exception:
                        print("Error encountered while loading <%s>"
                              % os.path.join(root, afile))
    print('** Loaded %s models, got controlled errors for %s files'
          % (ok, err))


if __name__ == '__main__':
    run_tests()



#! /usr/bin/python

import math

def chi_squared(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/(max(a1)-min(a1))**2/len(a1))

def L1_error_norm(a1,a2):

    import math

    abs_diff = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(abs_diff)/len(a1)

def error_norm(a1,a2,test_name='L1'):

    if 'L1'==test_name:
        return L1_error_norm(a1,a2)
    elif 'chi2'==test_name:
        return chi_squared(a1,a2)
    else:
        raise NameError('I do not know test '+test_name)

def density_prof(x,t):

    return 1+1e-6*math.sin(2*math.pi*(x-t))

def pressure_prof(x,t):

    return 3./5.+1e-6*math.sin(2*math.pi*(x-t))

def velocity_prof(x,t):

    return 1e-6*math.sin(2*math.pi*(x-t))

def plot_primitives(x_list,
                    d_list,
                    p_list,
                    v_list,
                    d_exact,
                    p_exact,
                    v_exact):
    import pylab

    pylab.subplot(311)
    pylab.plot(x_list,d_list,
               x_list,d_exact)

    pylab.subplot(312)
    pylab.plot(x_list,p_list,
               x_list,p_exact)

    pylab.subplot(313)
    pylab.plot(x_list,v_list,
               x_list,v_exact)

    pylab.show()

def plot_primitives_dfr(x_list,
                        d_list,
                        p_list,
                        v_list,
                        d_exact,
                        p_exact,
                        v_exact):
    import pylab

    pylab.subplot(311)
    pylab.plot(x_list,d_list-d_exact)

    pylab.subplot(312)
    pylab.plot(x_list,p_list-p_exact)

    pylab.subplot(313)
    pylab.plot(x_list,v_list-v_exact)

    pylab.show()

def calc_entropy(d,p,g):

    return p/d**g

def calc_riemann_invariant(d,p,v,g,sgn):

    if abs(sgn)<>1:
        raise NameError('Sing must be either 1 or -1')

    c = (g*p/d)**0.5
    return 2*c/(g-1)+sgn*v

def calc_invariants(d_list,
                    p_list,
                    v_list,
                    g):
    
    entropy = [calc_entropy(d,p,g) for d,p in zip(d_list,p_list)]
    jplus = [calc_riemann_invariant(d,p,v,g,1) for d,p,v in zip(d_list,p_list,v_list)]
    jminus = [calc_riemann_invariant(d,p,v,g,-1) for d,p,v in zip(d_list,p_list,v_list)]
    
    return entropy, jplus, jminus

def plot_invariants(x_list,
                    d_numeric,
                    p_numeric,
                    v_numeric,
                    d_exact,
                    p_exact,
                    v_exact,
                    g):

    s_n, jp_n, jm_n = calc_invariants(d_numeric,
                                      p_numeric,
                                      v_numeric, g)

    s_e, jp_e, jm_e = calc_invariants(d_exact,
                                      p_exact,
                                      v_exact, g)

    import pylab

    pylab.subplot(311)
    pylab.plot(x_list, s_n,
               x_list, s_e)

    pylab.subplot(312)
    pylab.plot(x_list, jp_n,
               x_list, jp_e)

    pylab.subplot(313)
    pylab.plot(x_list, jm_n,
               x_list, jm_e)

    pylab.show()

def main():

    import os

    import numpy

    t = numpy.loadtxt('time.txt')
    x_list = numpy.loadtxt('cell_centres.txt')
    d_list = numpy.loadtxt('densities.txt')
    p_list = numpy.loadtxt('pressures.txt')
    v_list = numpy.loadtxt('velocities.txt')
    da = [density_prof(x,t) for x in x_list]
    pa = [pressure_prof(x,t) for x in x_list]
    va = [velocity_prof(x,t) for x in x_list]

    if False:
        plot_primitives(x_list,
                        d_list,
                        p_list,
                        v_list,
                        da,pa,va)

    if False:
        plot_primitives_dfr(x_list,
                            d_list,
                            p_list,
                            v_list,
                            da,pa,va)

    if False:
        plot_invariants(x_list,
                        d_list,
                        p_list,
                        v_list,
                        da,pa,va,5./3.)

    test_name = 'L1'
    gof1 = error_norm(d_list,da,test_name)
    gof2 = error_norm(p_list,pa,test_name)
    gof3 = error_norm(v_list,va,test_name)

    f = open('gradesheet.txt','w')
    f.write(str(gof1)+'\n')
    f.write(str(gof2)+'\n')
    f.write(str(gof3)+'\n')
    f.close()

    return gof1<0.06 and gof2<0.02 and gof3 < 0.04

import sys

if __name__=='__main__':

    print main()



#! /usr/bin/python

def all_equal(ar):
    """
    Checks that all terms in the array are equal
    Input:
    ar - Numerical array
    """

    for i in ar:
        if i!=ar[0]:
            return False
    return True

def main():

    import numpy

    mass, xmom, ymom, enr, tracer = \
        numpy.loadtxt('res.txt',unpack=True);

    f = open('gradesheet.txt','w')
    f.write('mass '+str(all_equal(mass))+'\n')
    f.write('xmom '+str(all_equal(xmom))+'\n')
    f.write('ymom '+str(all_equal(ymom))+'\n')
    f.write('enr '+str(all_equal(enr))+'\n')
    f.write('tracer '+str(all_equal(tracer))+'\n')
    f.close()

    return all_equal(mass) and \
        all_equal(xmom) and \
        all_equal(ymom) and \
        all_equal(enr) and \
        all_equal(tracer)

if __name__=='__main__':
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



FORMATS = ["CSM", 
            "LWS", 
            "B3D", 
            "COB", 
            "PLY", 
            "IFC", 
            "OFF", 
            "SMD", 
            "IRRMESH", 
            "3D", 
            "DAE", 
            "MDL", 
            "HMP", 
            "TER", 
            "WRL", 
            "XML", 
            "NFF", 
            "AC", 
            "OBJ", 
            "3DS", 
            "STL", 
            "IRR", 
            "Q3O",
            "Q3D",
            "MS3D", 
            "Q3S", 
            "ZGL", 
            "MD2", 
            "X", 
            "BLEND", 
            "XGL", 
            "MD5MESH", 
            "MAX", 
            "LXO", 
            "DXF", 
            "BVH", 
            "LWO",
            "NDO"]

def available_formats():
    return FORMATS



def l1_norm(a1, a2):

    return sum([abs(x-y) for x,y in zip(a1, a2)])/len(a1)

def main(graphic_flag=False):

    import h5py
    import numpy
    import imp
    import os
    afd = imp.load_source('afd',os.environ['RICH_ROOT']+'/analytic/afd.py')
    import argparse

    initial = h5py.File('initial.h5')
    final_numeric = h5py.File('final.h5')

    init_parsed = {}
    init_parsed['grid'] = initial['grid']
    init_parsed['adiabatic index'] = numpy.loadtxt('adiabatic_index.txt')
    init_parsed['ambient'] = {}
    init_parsed['ambient']['density'] = numpy.loadtxt('ambient_density.txt')
    init_parsed['ambient']['pressure'] = numpy.loadtxt('ambient_pressure.txt')
    init_parsed['ambient']['velocity'] = numpy.loadtxt('ambient_velocity.txt')
    init_parsed['pert'] = {}
    init_parsed['pert']['density'] = [d-init_parsed['ambient']['density']
                                      for d in initial['density']]
    init_parsed['pert']['pressure'] = [d-init_parsed['ambient']['pressure']
                                       for d in initial['pressure']]
    init_parsed['pert']['velocity'] = [d-init_parsed['ambient']['velocity']
                                       for d in initial['x_velocity']]
    time = numpy.array(final_numeric['time'])[0]

    final_exact = afd.exact_time_advance(init_parsed, time)
    final_1o = afd.first_order_time_advance(init_parsed, time)

    if graphic_flag:

        import pylab
        import matplotlib

        for i, field in enumerate(['density',
                                   'pressure',
                                   'velocity']):
            pylab.subplot(3,1,i+1)
            pylab.gca().ticklabel_format(axis='y',style='sci',
                                         scilimits=(-2,2))
            pylab.tick_params(axis='x',
                              which='both',
                              bottom='off',
                              labelbottom='off')
            pylab.plot(initial['grid'],
                       init_parsed['pert'][field])
            pylab.plot(final_numeric['grid'],
                       [var - init_parsed['ambient'][field] for var in 
                        final_numeric[field.replace('velocity','x_velocity')]],
                       'k')
            pylab.plot(initial['grid'],
                       final_1o[field], linewidth=3, alpha=0.5)
            pylab.plot(initial['grid'], final_exact[field], 'r')
            pylab.ylabel(field)
        pylab.tick_params(axis='x',
                          which='both',
                          bottom='on',
                          labelbottom='on')
        pylab.xlabel('x')
        pylab.show()

    l1_list = {}
    for field in final_1o:
        l1_list[field] = l1_norm([init_parsed['ambient'][field]+var
                                  for var in final_1o[field]],
                                 final_numeric[field.replace('velocity','x_velocity')])

    f = open('gradehseet.txt','w')
    for field in ['density', 'pressure', 'velocity']:
        f.write(str(l1_list[field])+'\n')
    f.close()
        
    return abs(l1_list['density'])<2e-6 and \
        abs(l1_list['pressure'])<2e-6 and \
        abs(l1_list['velocity'])<2e-6

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




#! /usr/bin/python

import matplotlib
matplotlib.use('Qt4Agg')
import pylab
import numpy
import sys
import imp
sedov_taylor = imp.load_source('sedov_taylor','../analytic/sedov_taylor.py')

# Simulation results

pref = sys.argv[0].replace('draw_rad_prof.py','')

rawd = numpy.loadtxt(pref+'rad_prof.txt')
rr = rawd[:,0]
dr = rawd[:,1]
pr = rawd[:,2]
vr = rawd[:,3]

rs = numpy.sort(rr)
ids = numpy.argsort(rr)
ds = [dr[i] for i in ids]
ps = [pr[i] for i in ids]
vs = [vr[i] for i in ids]

# Analytic results
R = rs[numpy.argmax(ps)]
vatsf = vs[numpy.argmax(ps)]
patsf = ps[numpy.argmax(ps)]
datsf = ds[numpy.argmax(ps)]
g = 5./3.
w = 0
n = 2
v = numpy.linspace(1e-6+1./g,2/(g+1),num=1000)
ra = [R*sedov_taylor.vtoz(i,w,g,n) for i in v]
da = [datsf*sedov_taylor.vtod(i,w,g,n)/((g+1)/(g-1)) for i in v]
pa = [patsf*sedov_taylor.vtop(i,w,g,n)/(2/(g+1)) for i in v]
va = [vatsf*sedov_taylor.vtoz(i,w,g,n)*i/(2/(g+1)) for i in v]

# Show data
pylab.subplot(311)
pylab.plot(rs,ds,ra,da)
pylab.xlabel('r')
pylab.ylabel('Density')

pylab.subplot(312)
pylab.plot(rs,ps,ra,pa)
pylab.xlabel('r')
pylab.ylabel('Pressure')

pylab.subplot(313)
pylab.plot(rs,vs,ra,va)
pylab.xlabel('r')
pylab.ylabel('Velocity')

pylab.show()



import os
import re
import subprocess
import sys

from skbuild import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="vivid3d",
    version="0.2.0",
    packages=['vivid3d'],
    package_dir={'': 'src'},
    cmake_install_dir='src/vivid3d',
    author='GalaxyHuntersIL',
    author_email='galaxyhuntersil@gmail.com',
    url='https://galaxyhuntersil.wixsite.com/labs/vivid',
    description='A 3D visualisation tool for simulations',
    long_description=long_description,
    long_description_content_type='text/markdown',
    zip_safe=False,
    extras_require={"test": ["pytest>=6.0"]},
    python_requires=">=3.6"
)



#! /usr/bin/python

import numpy

def is_increasing(ar):
    """
    Checks that the array is in increasing order
    Input:
    ar - Numerical array
    """

    temp = numpy.diff(ar)
    for i in temp:
        if i<0:
            return False
    return True

def is_decreasing(ar):
    """
    Checks that the array is in decreasing order
    Input:
    ar - Numerical array
    """

    temp = numpy.diff(ar)
    for i in temp:
        if i>0:
            return False
    return True

def main():

    rawd = numpy.loadtxt('res.txt');
    return is_increasing(rawd[:,0]) and \
        is_decreasing(rawd[:,1])

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



def make_temp_folder(pref,uname):

    import re

    res = pref+'_'+uname
    res = res.replace('.','')
    res = res.replace('/','_')
    res = res.replace('__','_')
    res = re.sub(r'_$','',res)
    return res

def invalid_syntax_announcement():
    print 'Invalid syntax'
    print 'proper syntax:'
    print 'python ./convergence_curve.py [test folder] [low res] [high res] [res step] [output file]'
    print 'exapmple use:'
    print 'python ./convergence_curve.py ./acoustic_2d/base/ 20 101 10 l1_vs_res.txt'

def main():

    import argparse
    import os

    parser = argparse.ArgumentParser(description='Generates convergence curves')
    parser.add_argument('test_folder',
                        help='path to test folder')
    parser.add_argument('lowest_res',
                        type=int,
                        help='lowest bound on res (power of 2)')
    parser.add_argument('highest_res',
                        type=int,
                        help='upper bound on res (power of 2)')
    parser.add_argument('output_file',
                        help='file to write the data to')
    args = parser.parse_args()

    os.system('python ./new_run_tests.py '+args.test_folder)
    exec_folder = make_temp_folder('temp',args.test_folder)

    os.system('python ./prep_dif_res_runs.py '+exec_folder+' '+
              str(args.lowest_res)+' '+str(args.highest_res))

    os.system('python ./execute_all_runs.py '+exec_folder+'_res_* parallel')

    os.system('python ./collect_data.py '+exec_folder+'_res_* '+args.output_file)
    
if __name__=='__main__':
    
    main()



def execute_single(folder_path):

    """
    Executes a single simulation run
    Input:
    folder_path - Path to the run
    """

    import os
    home = os.getcwd()
    os.chdir(folder_path)
    os.system('./test.exe')
    os.system('python ./test.py')
    os.chdir(home)

def execute_all_runs(folder_list, pflag):

    """
    Executes all the simulations
    ptrn - Pattern of folder names
    pflag - Parallel flag (toggles using parallelisation)
    """

    if pflag:

        import multiprocessing
        pool = multiprocessing.Pool()
        pool.map(execute_single,folder_list)
    else:
        [execute_single(folder) for folder in folder_list]

def inalid_syntax_announcement():

    print 'Invalid syntax. Proper syntax:'
    print 'python ./execute_all_runs.py [folder list] [serial/parallel]'
    print 'for example:'
    print 'python ./execute_all_runs.py temp_acoustic_2d_res_* parallel'
    print 'will run all acoustic_2d runs in parallel'

if __name__=='__main__':
    
    import sys

    folder_list = sys.argv[1:-1]
    if sys.argv[-1]=='parallel':
        parallel_flag = True
    elif sys.argv[-1]=='serial':
        parallel_flag = False
    else:
        raise NameError('Unknown parallelisation option '+sys.argv[-1])

    execute_all_runs(folder_list,parallel_flag)



#! /usr/bin/python

def l1_fit_factor(a1, a2):

    import math

    diff2 = [abs(x-y) for x,y in zip(a1,a2)]
    return sum(diff2)/len(a1)

def main():

    import numpy
    import os
    #import imp
    import math

    x_list, d_list, p_list, vx_list, vy_list = \
        numpy.loadtxt('prof1d.txt', unpack=True)

    rho_0 = numpy.loadtxt('mean_density.txt')
    amp = numpy.loadtxt('amplitude.txt')
    k = 2*numpy.pi/numpy.loadtxt('wavelength.txt')
    vp = numpy.loadtxt('phase_velocity.txt')
    p_0 = numpy.loadtxt('mean_pressure.txt')
    gamma = numpy.loadtxt('adiabatic_index.txt')
    c_0 = numpy.sqrt(gamma*p_0/rho_0)
    t = numpy.loadtxt('time.txt')

    def density_exact(x):

        return rho_0-\
            amp*rho_0*math.cos(k*x)*\
            (vp*math.sin(c_0*k*t)-c_0*math.sin(vp*k*t))/\
            (k*c_0*(c_0-vp)*(c_0+vp))

    def pressure_exact(x):

        return p_0-\
            amp*c_0*rho_0*math.cos(k*x)*\
            (vp*math.sin(c_0*k*t)-c_0*math.sin(k*t*vp))/\
            (k*(c_0-vp)*(c_0+vp))

    def velocity_exact(x):

        return -amp*vp*math.sin(k*x)*\
            (math.cos(k*t*vp)-math.cos(k*t*c_0))/\
            (k*(c_0-vp)*(c_0+vp))

    d_exact = [density_exact(x)
               for x in x_list]
    p_exact = [pressure_exact(x)
               for x in x_list]
    v_exact = [velocity_exact(x)
               for x in x_list]

    if False:
        import pylab
        pylab.subplot(311)
        pylab.plot(x_list, d_list, '.')
        pylab.plot(x_list, d_exact, '.')
        pylab.subplot(312)
        pylab.plot(x_list, p_list, '.')
        pylab.plot(x_list, p_exact, '.')
        pylab.subplot(313)
        pylab.plot(x_list, vx_list, '.')
        pylab.plot(x_list, v_exact, '.')
        pylab.show()

    l1_density = l1_fit_factor(d_list, d_exact)
    l1_pressure = l1_fit_factor(p_list, p_exact)
    l1_velocity = l1_fit_factor(vx_list, v_exact)

    f = open('gradesheet.txt','w')
    f.write(str(l1_density)+'\n')
    f.write(str(l1_pressure)+'\n')
    f.write(str(l1_velocity)+'\n')
    f.close()
    
    return l1_density < 1.27e-5 and \
        l1_pressure < 2.2e-5 and \
        l1_velocity < 3.4e-5

import sys
if __name__=='__main__':
    print main()



def extract_contour(x_list, y_list, z_list, z_value):

    import matplotlib
    matplotlib.use('Agg')
    import pylab
    import numpy

    temp = pylab.tricontour(x_list, y_list, z_list, levels=[z_value])
    pylab.clf()
    return numpy.concatenate([itm.vertices for itm
                              in temp.collections[0].get_paths()])

def consolidate_single(fname):

    import h5py
    import numpy

    res = {}
    with h5py.File(fname,'r+') as f:
        structure = {'geometry':['x_coordinate','y_coordinate'],
                     'hydrodynamic':['density','pressure'],
                     'stickers':['wedge']}
        for field in structure:
            for subfield in structure[field]:
                res[subfield] = numpy.array(f[field][subfield])
    return res

def consolidate_multiple(fname_list):

    import numpy
    
    part_list = [consolidate_single(fname) for fname in fname_list]
    res = {}
    for field in part_list[0]:
        res[field] = numpy.concatenate([p[field] for p in part_list])
    return res

def main():

    import numpy
    import math
    import imp
    import os
    oblique_shock = imp.load_source(\
        'oblique_shock',\
            os.environ['RICH_ROOT']+'/analytic/oblique_shock.py')
    import h5py
    import glob

    if len(glob.glob('process_*_final.h5'))>0:
        raw = consolidate_multiple(glob.glob('process_*_final.h5'))
    else:
        raw = consolidate_single('final.h5')
    numeric = {}
    for field in raw:
        numeric[field] = raw[field][raw['wedge']<0.5]
    wedge_angle = numpy.loadtxt('wedge_angle.txt')
    adiabatic_index =numpy.loadtxt('adiabatic_index.txt')
    mach = numpy.loadtxt('mach_number.txt')
    v_th = 0.3
    numeric['entropy'] = numeric['pressure']/numeric['density']**adiabatic_index
    shock_front = extract_contour(numeric['x_coordinate'],
                                  numeric['y_coordinate'],
                                  numeric['entropy'],
                                  1.1)
    x_far = shock_front.T[0][shock_front.T[0]>0]
    y_far = shock_front.T[1][shock_front.T[0]>0]
    lfit = numpy.polyfit(x_far, y_far,1)
    shock_angle = numpy.arctan(lfit[0])
    exact_shock_angle = oblique_shock.calc_shock_angle\
        (mach, adiabatic_index, wedge_angle)

    difrat = abs(shock_angle-exact_shock_angle)\
        /exact_shock_angle
    
    f = open('gradesheet.txt','w')
    f.write(str(difrat)+'\n')
    f.close()

    return difrat<0.4

if __name__=='__main__':
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')



#! /usr/bin/python

def main():

    import numpy
    pa = 0.224614
    va = 0
    rawd = numpy.loadtxt('res.txt')
    pn = rawd[0]
    vn = rawd[1]

    f = open('report.txt','w')
    f.write('Analytic pressure = '+str(pa)+'\n')
    f.write('Numeric pressure = '+str(pn)+'\n')
    f.write('Analytic velocity = '+str(va)+'\n')
    f.write('Numeric velocity = '+str(vn)+'\n')
    f.close()

    return abs(pa-pn)/(pa+pn)<0.02 and abs(vn)<0.01

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




#! /usr/bin/python

def main():

    import numpy

    np_list = numpy.loadtxt('np_list.txt')
    cc_pcm = numpy.loadtxt('cc_pcm.txt')
    cc_plm = numpy.loadtxt('cc_plm.txt')

    graphic_flag = False
    if graphic_flag:

        import pylab

        rawd_pcm = numpy.loadtxt('pcm_20_profile.txt')
        rawd_plm = numpy.loadtxt('plm_20_profile.txt')
        pylab.plot(rawd_pcm[:,0],rawd_pcm[:,1],
                   rawd_pcm[:,0],rawd_pcm[:,2],
                   rawd_plm[:,0],rawd_plm[:,2])
        pylab.show()

        pylab.loglog(np_list,cc_pcm,label='PCM')
        pylab.loglog(np_list,cc_plm,label='PLM')
        pylab.show()

    pcm_power = numpy.polyfit(numpy.log(np_list),numpy.log(cc_pcm),1)[0]
    plm_power = numpy.polyfit(numpy.log(np_list),numpy.log(cc_plm),1)[0]

    f = open('gradesheet.txt','w')
    f.write(str(pcm_power)+'\n'+
            str(plm_power)+'\n')
    f.close()

    return abs(pcm_power-(-1))<0.01 and \
        abs(plm_power-(-2))<0.01

if __name__=='__main__':
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')




#! /usr/bin/python

def main():

    import numpy
    
    rawd = numpy.loadtxt('res.txt')
    return rawd[0]==rawd[1]

if __name__=='__main__':
    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')





#! /usr/bin/python

def IsPureVirtual(fname):
    """
    Returns true if a header file is a pure virtual (no implementation)
    Pure virtual header files are designated as such by the phrase "Pure virtual"
    at the first line
    Input:
    fname - File name
    """

    f = open(fname)
    fl = f.readline()
    f.close()
    return fl.find('Pure virtual')>-1

def RemoveFileFromPath(fpath):
    """
    Removes a file name from the path
    Input:
    fpath - Path to file
    """

    if fpath.find('/')>-1:
        rfpath = fpath[::-1]
        rfpathnf = rfpath[rfpath.find('/'):]
        fpathnf = rfpathnf[::-1]
        res = fpathnf

        return res
    else:
        return ''

def JustFileName(fpath):
    """
    Extracts the file name from the path
    Input:
    fpath - Full path
    """

    if(fpath.find('/')>-1):
        rfpath = fpath[::-1]
        rfname = rfpath[:rfpath.find('/')]
        fname = rfname[::-1]
        return fname
    else:
        return fpath

def JoinPaths(apath, rpath):
    """
    Joins a relative path to an absolute path
    Input:
    apath - Absolute path
    rpath - Relative path
    """

    if len(rpath)==0:
        res = apath
    elif len(apath)==0:
        res = './'+rpath
    elif rpath[0]=='/' or apath[-1]=='/':
        res = apath+rpath
    else:
        res = apath+'/'+rpath
    while(res.find('..')>-1):
        aux1 = res.split('/')
        aux2 = aux1.index('..')
        aux3 = aux1[aux2-1]
        res = res.replace(aux3+'/../','')
    return res

class CppInterpreter:
    """
    Interprets c++ code
    """

    def __init__(self, cname, cflags, lflags=''):
        """
        Class constructor
        Input:
        cname - Compiler name, i.e. string used to invoke compiler
        cflag - Compilation flags
        """
        self.__cname = cname
        self.__cflags = cflags
        self.__lflags = lflags

    def SimpleCompile(self, sfname):
        """
        Creates an object file from a source file
        Input:
        sfname - Name of the source file
        """

        import os
        ef = os.system(self.__cname+' '+self.__cflags+' -c '+sfname)
        if ef!=0:
            raise NameError('Failed to compile')

    def NeedCompile(self,hfname):
        """
        Checkes whether a certain file needs to be compiled
        Returns true if so, and false if the file has already
        been compile (if a newer object file exists)
        """

        if 'treecode' in hfname:
            return False

        res = 1
        if hfname[-4:]!='.hpp':
            print 'NeedCompile only accepts files with the extension .hpp'
            print 'Input file is '+hfname
            raise NameError('NotHeader')
        import os
        if os.path.exists(JustFileName(hfname).replace('.hpp','.o')):
            t_hpp = os.path.getmtime(hfname)
            t_o = os.path.getmtime(JustFileName(hfname).replace('.hpp','.o'))
            t_cpp = os.path.getmtime(hfname.replace('.hpp','.cpp'))
            if (t_o>t_cpp) and (t_o>t_hpp):
                res = 0
        return res                                   

    def RecursiveCompile(self, sfname):
        """
        Checks a source file for dependencies and compiles them recursively
        """

        f = open(sfname)
        for i in f:
            if i.find('#include')>-1 and i.find('"')>i.find('#define'):
                hname = i.split('"')[1]
                fpath = JoinPaths(RemoveFileFromPath(sfname),
                                  RemoveFileFromPath(hname))
                fname = hname.replace('hpp','cpp')
                if self.NeedCompile(fpath+JustFileName(hname)) and \
                        JustFileName(sfname)!=fname:
                    self.RecursiveCompile(fpath+JustFileName(hname))
                    if not IsPureVirtual(fpath+JustFileName(hname)):
                        self.RecursiveCompile(fpath+JustFileName(fname))
        f.close()
        if(sfname[-4:]=='.cpp'):
            self.SimpleCompile(sfname)

    def Link(self, sfname):
        """
        Links object files to an executable
        Input:
        sfname - Name of main file
        To - Do: The function assumes the input file has the extension .cpp
        At the moment I don't think that in this project I would need other
        extensions, so I decided to raise an error in case a file with a
        different extension is entered
        """

        if sfname.find('.cpp')==-1:
            raise NameError('automaker currently supports only .cpp files')

        import os
        ef = os.system(self.__cname+' -o '+sfname.replace('.cpp','.exe')+' *.o'+
                       ' '+self.__lflags)
        if ef!=0:
            raise NameError('Failed to link '+sfname.replace('.cpp','.exe'))

    def Run(self, sfname):
        """
        Runs the executable
        This function assumes the executable was created using the Link function
        Input:
        sfname - Name of the main source file
        """

        if sfname.find('.cpp')==-1:
            raise NameError('automaker currently supports only .cpp files')

        import os
        os.system('./'+sfname.replace('.cpp','.exe'))

    def Interpret(self, sfname):
        """
        Compiles, links and executes
        Input:
        sfname - Name of the main source file
        """

        if sfname.find('.cpp')==-1:
            raise NameError('automaker currently supports only .cpp files')

        self.RecursiveCompile(sfname)
        self.Link(sfname)
        self.Run(sfname)

import sys
if __name__=="__main__":
    fname = sys.argv[1]

    import os
    os.system('rm *.o *.exe')
    myinterp = CppInterpreter('g++','')
    myinterp.Interpret(fname)



#! /usr/bin/python

import math

def goodness_of_fit(a1, a2):

    import math

    diff2 = [(x-y)**2 for x,y in zip(a1,a2)]
    return math.sqrt(sum(diff2)/(max(a1)-min(a1))**2/len(a1))

def density_prof(x):

    if x<0.3 or x>0.7:
        return 1
    else:
        return 2

def pressure_prof(x):

    return 1

def velocity_prof(x):

    return 1

def plot_fluxes():

    import numpy
    import pylab

    rawd = numpy.loadtxt('fluxes.txt')
    radius = rawd[:,0]
    mass_flux = rawd[:,1]
    momentum_flux = rawd[:,2]
    energy_flux = rawd[:,3]
    
    pylab.subplot(311)
    pylab.plot(radius,mass_flux)

    pylab.subplot(312)
    pylab.plot(radius,momentum_flux)

    pylab.subplot(313)
    pylab.plot(radius,energy_flux)

    pylab.show()

def plot_hydro_vars():

    import numpy
    import pylab

    for int_name in ['pcm','plm_naive','plm']:
        x_list = numpy.loadtxt(
            int_name+'_cell_centres.txt')
        d_list = numpy.loadtxt(
            int_name+'_densities.txt')
        p_list = numpy.loadtxt(
            int_name+'_pressures.txt')
        v_list = numpy.loadtxt(
            int_name+'_velocities.txt')

        pylab.subplot(311)
        pylab.plot(x_list,d_list)
        pylab.ylabel('Density')
        
        pylab.subplot(312)
        pylab.plot(x_list,p_list)
        pylab.ylabel('Pressure')
        
        pylab.subplot(313)
        pylab.plot(x_list,v_list)
        pylab.ylabel('Velocity')
        pylab.xlabel('Radius')

    da = [density_prof(x) for x in x_list]
    pa = [pressure_prof(x) for x in x_list]
    va = [velocity_prof(x) for x in x_list]

    pylab.subplot(311)
    pylab.plot(x_list,da)

    pylab.subplot(312)
    pylab.plot(x_list,pa)

    pylab.subplot(313)
    pylab.plot(x_list,va)

    pylab.show()
        
def main():

    import os
    import numpy
    import h5py

    if False:
        return False

    if False:
        plot_fluxes()

    if False:
        plot_hydro_vars()

    pcm_data = h5py.File('pcm_final.h5')
    plm_data = h5py.File('plm_final.h5')
    x_list = pcm_data['grid']
    d_list_pcm = pcm_data['density']
    d_list_plm = plm_data['density']
    da = [density_prof(x) for x in x_list]
    gof_pcm = goodness_of_fit(d_list_pcm,da)
    gof_plm = goodness_of_fit(d_list_plm,da)

    f = open('gradesheet.txt','w')
    f.write(str(gof_pcm)+'\n')
    f.write(str(gof_plm)+'\n')
    f.close()

    return gof_plm<0.12 and \
        gof_pcm<0.22 and \
        gof_plm<gof_pcm

if __name__=='__main__':

    import os

    if main():
        os.system('touch test_passed.res')
    else:
        os.system('touch test_failed.res')

